/*
 * Implementa�o do algoritmo: Recursive MLSL-AP-EF
 *
 * - Number of reference Signals
 * - Number of output Signals
 * - Lattice Order
 * - Forgeting Factor;
 * - Inicialization
 *
 * As entradas s�:
 * - u(n), reference signals
 *  - d(n), desired signals
 *
 * As saidas s�:
 * - e(n)=d(n)-d^(n|Un), the a posteriori estimation error
 *
 * Paulo Lopes 17-12-2001
 */

#include <math.h>
#include <memory.h> 
#include <assert.h>
#include <stdio.h>
#include "McLatticePr.h"

/* Lattice arguments
*
* - Number of reference Signals
* - Number of output Signals
* - Lattice Order
* - Forgeting Factor;
* - Inicialization
* - Sample Time
* - Sample time of internal state output updates
*/

class MLSLattice: public McLatticePr {
public:
	MyData lambda;
	
	MyData *Bi1p;
	MyData *Fi1p;
	MyData *GammaF1p;
	MyData *GammaB1p;
	MyData *gamma1p;
	MyData *kappa1p;
	
	MyData *Bi0p;
	MyData *Fi0p;
	MLSLattice(int inSize, int outSize, int order, double lambda, double ini);
	~MLSLattice(void);
	//Update function
	void Update(val_type *in, val_type *d);
	//Output function
	void Output(val_type *out);

	//Fun�es auxiliares
	MyData &Bi0(int m, int i, int j)		
	{ assert(m<=order && i<inSize && j<inSize); return Bi0p[m*inSizeSq+j*inSize+i]; }
	MyData &Fi0(int m, int i, int j)		
	{ assert(m<=order && i<inSize && j<inSize); return Fi0p[m*inSizeSq+j*inSize+i]; }
	MyData &Bi1(int m, int i, int j)		
	{ assert(m<=order && i<inSize && j<inSize); return Bi1p[m*inSizeSq+j*inSize+i]; }
	MyData &Fi1(int m, int i, int j)		
	{ assert(m<=order && i<inSize && j<inSize); return Fi1p[m*inSizeSq+j*inSize+i]; }
	MyData &GammaF1(int m, int i, int j)		
	{ assert(m<=order && i<inSize && j<inSize); return GammaF1p[m*inSizeSq+j*inSize+i]; }
	MyData &GammaB1(int m, int i, int j)		
	{ assert(m<=order && i<inSize && j<inSize); return GammaB1p[m*inSizeSq+j*inSize+i]; }
	MyData &gamma1(int m)
	{ assert(m<=order); return gamma1p[m]; }
	MyData &kappa1(int m, int i, int j)
	{
		assert(m<=order); assert(i<outSize); assert(j<inSize);
		return kappa1p[m*outSize*inSize+j*outSize+i];
	}

	double time;
};
